@@ -71,6 +71,11 @@
 #define PNG_READ_STRIP_ALPHA_SUPPORTED
 #define PNG_READ_EXPAND_SUPPORTED
 #define PNG_READ_FILLER_SUPPORTED
+#define PNG_READ_PACK_SUPPORTED
+#define PNG_READ_SHIFT_SUPPORTED
+
+#define PNG_WRITE_PACK_SUPPORTED
+#define PNG_WRITE_SHIFT_SUPPORTED
 
 #if (PNGCRUSH_LIBPNG_VER > 10002)
 /* versions 0.96 through 1.0.2 have a stub png_rgb_to_gray() with the
